varying vec3 derivativeIn;
varying float structural_abstraction;
varying float illustrativeness;

// Pastel colors
varying vec3 pastel_light;
varying vec3 pastel_dark;

vec4 color() {
	float gradient;
	float r2;

	if (structural_abstraction <= 0.75) {
		if (structural_abstraction < 0.75) {
			r2 = 1.0 - gl_TexCoord[0].y * gl_TexCoord[0].y;
			gradient = mix(r2, 1.0, structural_abstraction / 0.75);
			return gl_Color * gradient;
		}
		else {
			return gl_Color;
		}
	} else {
		/* Alternate color for inside of helix */
		if (!gl_FrontFacing || gl_TexCoord[0].p < 0.5)
			return gl_Color;
		else
			return 0.5 * gl_Color;
	}
}

vec4 pastel() {
	float gradient;
	float r2;
	float tmp;

	if (structural_abstraction < 0.6) {
		/* Look like a cylinder */
		r2 = 1.0 - gl_TexCoord[0].y * gl_TexCoord[0].y;
		gradient = floor(r2 + 0.7);
		return vec4(mix(pastel_dark, pastel_light, gradient), 1.0);
	}
	else if (structural_abstraction < 0.75) {
		r2 = 1.0 - gl_TexCoord[0].y * gl_TexCoord[0].y;
		tmp = (0.75 - structural_abstraction) / (0.75 - 0.6);
		gradient = mix(1.0, floor(r2 + 0.7), tmp);
		return vec4(mix(pastel_dark, pastel_light, gradient), 1.0);
	}
	else {
		/* Alternate color for inside of helix and outlines */
		if (!gl_FrontFacing || gl_TexCoord[0].p < 0.5)
			return vec4(pastel_light, 1.0);
		else
			return vec4(pastel_dark, 1.0);
	}
}

vec4 grey() {
	vec4 col;
	float grey;

	col = normalize(pastel());
	grey = (0.30 * col.r + 0.59 * col.g + 0.11 * col.b);
	return vec4(grey);
}

vec4 hatch(float s) {
	vec4 helix;
	float blend;

  if (gl_TexCoord[0].p < 0.5)
		/* Display connecting splines as black lines */
    return vec4(0.0);
  else
		if (s > 0.75)
			/* Create an outline around the ribbon */
			return vec4(0.0);
		else {
			blend = (structural_abstraction - 0.75) / 0.25;
			/* Alternate helices between white and striped */
			if (!gl_FrontFacing)
				helix = vec4 (1.0);
			else
			  helix = vec4(step(0.5, fract(2.0 * 16.0/15.0 * gl_TexCoord[0].t)));
			return mix(vec4(0.0), helix, blend);
		}
}

void main(void)
{
	float s;
	float blend;

	s = abs(gl_TexCoord[0].s);
	/* Halo */
	if (s > 1.0) {
		gl_FragColor = vec4(1.0);
		gl_FragDepth = gl_FragCoord.z - (1.0 - s);
	}
	else {
		if (illustrativeness <= (1.0 / 3.0)) {
			blend = 3.0 * illustrativeness;
			gl_FragColor = mix(color(), pastel(), blend);
		}
		else if (illustrativeness <= (2.0 / 3.0)) {
			blend = 3.0 * (illustrativeness - (1.0 / 3.0));
			gl_FragColor = mix(pastel(), grey(), blend);
		}
		else {
			blend = 3.0 * (illustrativeness - (2.0 / 3.0));
			gl_FragColor = mix(grey(), hatch(s), blend);
		}
		gl_FragColor.a = 1.0;	
	}

	//gl_FragColor = vec4(derivativeIn, 1.0);
  //gl_FragColor = vec4(0.0, 0.0, 1.0, 1.0);
	gl_FragDepth = gl_FragCoord.z;
}
