#version 120
#extension GL_EXT_geometry_shader4 : enable

#define PROTSTART 1.5
#define PROTEND 4.0

uniform float mol_abstractionmax;
uniform float protein_radius;
uniform float depth_cueing;
uniform float attenuate;

varying in float structural_abs[];
varying out float structural_abstraction;
varying in float illustrativenessIn[];
varying out float illustrativeness;
varying in vec2 ranksIn[];
varying in vec3 directionIn[];
varying out float dist;
varying out vec3 plane_offset;
varying out vec3 plane_normal;
varying out vec4 col;
varying out float radius;
varying out float hint;
varying out float ds;
varying out float type;

float mol_abstraction;
float halo_radius;
float line_attenuation;

float endcap_present;
vec4 endcap_position;
vec3 endcap_plane_offset;
vec3 endcap_plane_normal;
vec4 endcap_color;

// Pastel colors
varying in vec3 pastel_light0In[];
varying in vec3 pastel_light1In[];
varying in vec3 pastel_dark0In[];
varying in vec3 pastel_dark1In[];
varying out vec3 pastel_light;
varying out vec3 pastel_dark;
varying in float nyIn[];
varying out float ny;
varying in float nzIn[];
varying out float nz;

varying in vec3 normal_1In[];
varying out vec3 normal_1;
varying in vec3 normal_2In[];
varying out vec3 normal_2;

varying in vec3 normalSIn[];
varying in vec3 normalPIn[];
varying out vec3 normalS;
varying out vec3 normalP;

varying out float ao_contrib;

// Debug meuk!
varying out vec3 direct;

mat4 coords = mat4(0.0,-1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0,-1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0);
mat4 coords_endcap = mat4(-1.0, -1.0, 0.0, 0.0, -1.0, 1.0, 0.0, 0.0, 1.0, -1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0);

void calculate_abstraction() {
	float tmp;
	float growth;
	float x;

	/* PRE: When this program is used: 0 <= structural_abs <= 0.6 */
	if (structural_abstraction < 0.25) {
		x = structural_abstraction * 4.0;
		growth = x * x * 0.8 + 0.2;
		tmp = structural_abstraction * (4.0 * 0.9);
		radius = growth * (1.5 / protein_radius) * (1.0 - tmp);
		mol_abstraction = mol_abstractionmax;
	}
	else {
		radius = (1.5 / protein_radius) * 0.1;
		tmp = 1.0/(0.6-0.25) * mol_abstractionmax * (structural_abstraction-0.25);
    mol_abstraction = mol_abstractionmax - tmp;
	}
}

vec4 stripOrientation(vec4 pos, vec4 coord, vec4 direction)
{
	float side;
	vec3 referenceVector;
  vec3 orig;
	vec3 refVec;
	float distCam;
	float attenuation;

	vec3 planeNormalProjection;
	vec3 offset;
	vec3 displacement;
	vec3 referenceProjection;
	vec3 shift;

	vec4 baseProj;
	vec4 dispProj;
	float depth_base;
	float depth_displacement;

	// Actually this is a bit nasty, since the position will change once we know ds!
	pos = pos + coord.s * direction;

	side = coord.t;
  orig = vec3(gl_ModelViewMatrixInverse * vec4(0.0,0.0,0.0,1.0));
	refVec = orig - vec3(pos);
  referenceVector = normalize(refVec);
  plane_offset = normalize(cross(vec3(direction),referenceVector));
  plane_normal = normalize(cross(plane_offset, directionIn[0]));
  dist = dot(radius * plane_normal, normalize(orig));

	planeNormalProjection = dot(plane_normal, referenceVector) * referenceVector; 
	offset = radius * (plane_normal - planeNormalProjection); // Just radius here?

	referenceProjection = dot(plane_normal, referenceVector) * plane_normal;
	// Now create the displacement vector and offset
	displacement = radius / length(referenceProjection) * referenceVector;
	offset = radius * plane_normal - displacement;
	ds = -1.0 * length(offset) / length(direction) * sign(dot(offset, vec3(direction)));

	if(dot(offset, vec3(direction)) * (coord.s - 0.5) > 0) {
	  shift = offset;
		coord.s += -1.0 * ds;
	} else {
	  shift = vec3(0.0);
		// Need to draw an end cap on this side
		endcap_present = 1.0;
		endcap_plane_offset = plane_offset * radius;
		endcap_plane_normal = plane_normal * radius;
		endcap_position = pos;
		endcap_color = col;
		hint = coord.s/2.0 + 0.5;
	}
	gl_TexCoord[0].t = coord.t * (1.0 + halo_radius);
	gl_TexCoord[0].s = coord.s;

	if( sign( (coord.s - 0.5) * ds ) < 0.0 ) {
	  gl_TexCoord[0].s -= ds;
	}

	// Do attenuation
	distCam = (length(refVec) - PROTSTART) / (PROTEND - PROTSTART);
	attenuation = mix(1.0, 1.5 * (1.0 - distCam) , line_attenuation);

	// Calculate depth
	pos = pos + vec4(shift, 0.0);
	baseProj = gl_ModelViewProjectionMatrix * pos;
	dispProj = gl_ModelViewProjectionMatrix * (pos + vec4(displacement, 0.0));
	depth_base = (baseProj.z / baseProj.w) / 2.0 + 0.5;
	depth_displacement = (dispProj.z / dispProj.w) / 2.0 + 0.5;
	dist = depth_displacement - depth_base;

  return pos + vec4(side * radius * attenuation * (1.0 + halo_radius) * plane_offset,0.0);
}

void depthOfTop()
{
  vec3 halfway;
  vec3 cam;
  float dist2;
  float depth_top;
  float depth_hw;
  vec4 top;
  vec4 halfway_proj;
  vec3 referenceVector;

  cam = vec3(gl_ModelViewMatrixInverse * vec4(0.0,0.0,0.0,1.0));
  halfway = vec3(gl_PositionIn[0]) + 0.5 * directionIn[0];
  referenceVector = normalize(cam - halfway);
  dist2 = dot(radius * plane_normal, referenceVector);
  top = gl_ModelViewProjectionMatrixInverse * vec4(dist2*referenceVector + halfway, 1.0);
  depth_top = (top.z / top.w) / 2.0 + 0.5;
  halfway_proj = gl_ModelViewProjectionMatrixInverse * vec4(halfway, 1.0);
  depth_hw = (halfway_proj.z / halfway_proj.w) / 2.0 + 0.5;
  dist = depth_top - depth_hw;
}

void main() {
	vec4 pos;
	vec4 dir;
	vec4 coord;
	vec3 direction;
	float s;

	endcap_present = 0.0;
	illustrativeness = illustrativenessIn[0];
  structural_abstraction = structural_abs[0];
  if (structural_abstraction > 0.6)
    return;

	nz = nzIn[0];
	ny = nyIn[0];
	normal_1 = normal_1In[0];
	normal_2 = normal_2In[0];
	direct = directionIn[0];

	normalS = normalSIn[0];
	normalP = normalPIn[0];

	if (depth_cueing < 0.5) {
		line_attenuation = 2.0 * depth_cueing * attenuate;
		halo_radius = 0.0;
	}
	else {
		line_attenuation = 1.0 * attenuate;
		halo_radius = 0.4 * (2.0 * depth_cueing - 1.0);
	}

	calculate_abstraction();

	if (mol_abstraction >= ranksIn[0][0]) {
		if (depth_cueing < 0.5)
			ao_contrib = 2.0 * depth_cueing;
		else
			ao_contrib = 1.0;

		for(int i = 0; i < 4; i++) {
			coord = coords[i];
			direction = directionIn[0];
			if (ranksIn[0][1] > mol_abstraction) {
				s = coord.s * (mol_abstraction - ranksIn[0][0]);
				direction *= (mol_abstraction - ranksIn[0][0]);
			}
			else {
				s = coord.s;
			}

			col = mix(gl_FrontColorIn[0], gl_FrontSecondaryColorIn[0], s);
			pastel_light = mix(pastel_light0In[0], pastel_light1In[0], s);
			pastel_dark = mix(pastel_dark0In[0], pastel_dark1In[0], s);
			type = 0.0;
			pos = stripOrientation(gl_PositionIn[0], coord,	vec4(direction, 0.0));
			gl_TexCoord[0].pq = gl_TexCoordIn[0][0].pq;
			gl_Position = gl_ModelViewProjectionMatrix * pos;
			EmitVertex();
		}
  }
	EndPrimitive();

	if(endcap_present > 0.5) {
		col = endcap_color;
		type = 1.0;
		for(int i = 0; i < 4; i++) {
			coord = coords_endcap[i];

			pos = endcap_position + vec4(coord.s * endcap_plane_offset, 0.0) + vec4(coord.t * endcap_plane_normal, 0.0);
			gl_TexCoord[0] = coord;
			gl_Position = gl_ModelViewProjectionMatrix * pos;
			EmitVertex();
		}
	}
}
