#version 120
#extension GL_EXT_geometry_shader4 : enable

uniform float mol_abstractionmax;
uniform float protein_radius;
uniform float structural_abstraction;

varying in vec2 ranksIn[];
varying in vec3 directionIn[];
varying out float dist;
varying out vec3 plane_offset;
varying out vec3 plane_normal;
varying out vec4 col;
varying out float radius;

float mol_abstraction;

mat4 coords = mat4(0.0,-1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0,-1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0);

void calculate_abstraction() {
	float tmp;

	/* PRE: When this program is used: 0 <= structural_abs <= 0.6 */
	if (structural_abstraction < 0.25) {
		tmp = structural_abstraction * (4.0 * 0.9);
		radius = 0.55 * (1.5 / protein_radius) * (1.0 - tmp);
		mol_abstraction = mol_abstractionmax;
	}
	else {
		radius = (1.5 / protein_radius) * 0.055;
		tmp = 1.0/(0.6-0.25) * mol_abstractionmax * (structural_abstraction-0.25);
    mol_abstraction = mol_abstractionmax - tmp;
	}
}

vec4 stripOrientation(vec4 pos, vec4 coord)
{
	float side;
	vec3 referenceVector;
  vec3 orig;

	side = coord.t;
  orig = vec3(gl_ModelViewMatrixInverse * vec4(0.0,0.0,0.0,1.0));
  referenceVector = normalize(orig - vec3(pos));
  plane_offset = normalize(cross(directionIn[0],referenceVector));
  plane_normal = normalize(cross(plane_offset, directionIn[0]));
  dist = dot(radius * plane_normal, normalize(orig));
  
  return pos + vec4(side * radius * plane_offset, 0.0);
}

void depthOfTop()
{
  vec3 halfway;
  vec3 cam;
  float dist2;
  float depth_top;
  float depth_hw;
  vec4 top;
  vec4 halfway_proj;
  vec3 referenceVector;

  cam = vec3(gl_ModelViewMatrixInverse * vec4(0.0,0.0,0.0,1.0));
  halfway = vec3(gl_PositionIn[0]) + 0.5 * directionIn[0];
  referenceVector = normalize(cam - halfway);
  dist2 = dot(radius * plane_normal, referenceVector);
  top = gl_ModelViewProjectionMatrixInverse * vec4(dist2*referenceVector + halfway, 1.0);
  depth_top = (top.z / top.w) / 2.0 + 0.5;
  halfway_proj = gl_ModelViewProjectionMatrixInverse * vec4(halfway, 1.0);
  depth_hw = (halfway_proj.z / halfway_proj.w) / 2.0 + 0.5;
  dist = depth_top - depth_hw;
}

void main() {
	vec4 pos;
	vec4 dir;
	vec4 coord;
	float s;

  //structural_abstraction = structural_abs[0];
  if (structural_abstraction > 0.6)
    return;

	calculate_abstraction();

	if (mol_abstraction >= ranksIn[0][0]) {
		for(int i = 0; i < 4; i++) {
			coord = coords[i];
			if (ranksIn[0][1] > mol_abstraction) {
				s = coord.s * (mol_abstraction - ranksIn[0][0]);
			}
			else {
				s = coord.s;
			}
			pos = stripOrientation(gl_PositionIn[0]
							+ vec4(directionIn[0] * s, 0.0), coord);
			gl_TexCoord[0] = coord;
			depthOfTop();
			gl_FrontColor = gl_FrontColorIn[0];
			
			gl_Position = gl_ModelViewProjectionMatrix * pos;
			EmitVertex();
		}
  }
}
