attribute vec3 derivative;
attribute vec3 linear_vertex;
attribute vec3 linear_derivative;
attribute vec3 helix_direction;
attribute float on_helix;
attribute float stripe_coord;

varying vec3 derivativeIn;
varying vec3 helix_directionIn;
varying float on_helixIn;
varying float stripe_coordIn;

attribute vec3 pastel_light;
varying vec3 pastel_lightIn;
attribute vec3 pastel_dark;
varying vec3 pastel_darkIn;

/* Structural abstraction lookup variables */
attribute float abstraction_lookup_pos;
uniform sampler1D abstraction_lookup;
varying float structural_abs;
varying float illustrativenessIn;

void main() {
	vec4 pos;
	vec3 deriv;
	float blend;

	structural_abs = texture1D(abstraction_lookup, abstraction_lookup_pos).r;
	illustrativenessIn = texture1D(abstraction_lookup, abstraction_lookup_pos).b;

  gl_TexCoord[0] = gl_MultiTexCoord0;
  gl_FrontColor = gl_Color;
  gl_BackColor = gl_Color;
	pastel_lightIn = pastel_light;
	pastel_darkIn = pastel_dark;

	if (structural_abs <= 0.6) {
		/* Linear cylinders */
		pos = vec4(linear_vertex, 1.0);
		deriv = linear_derivative;
	}
	else if (structural_abs < 0.75) {
		/* Transition from linear to splines */
		blend = 1.0 / (0.75 - 0.6) * (structural_abs - 0.6);
		pos = mix(vec4(linear_vertex, 1.0), gl_Vertex, blend);
		deriv = mix(linear_derivative, derivative, blend);
	}
	else {
		/* Splines */
		pos = gl_Vertex;
		deriv = derivative;
	}
	
	/* Moet straks zonder vermenigvulding met de matrix
	 * door naar de geometry-shader
	 */
	//gl_Position = gl_ModelViewProjectionMatrix * pos;
	gl_Position = pos;
	derivativeIn = deriv;
	helix_directionIn = helix_direction;
	on_helixIn = on_helix;
	stripe_coordIn = stripe_coord;
}
